#!/bin/bash
function myHelp () {
# Using a here doc with standard out.
cat <<-END

Help for $0

Backs up the RAMS application from the currently running tomcat web application in \$CATALINA_HOME\webapps and the 
configuration file from \$CATALINA_HOME\conf\Catalina\localhost directories, putting them in the \$CATALINA_HOME\backup
directory. The script verifies that the source files are where they are expected to be, stops tomcat if it is running, 
copies the files to the backup directory, removes the source files, copies the commons-codec file, and verifies that 
the script was successful.

Usage:
------
   $0 -h | --help
     Display this help
   $0 <RAMS-2.x.0>
     Run $0 to backup the RAMS-2.x.0 application and configuration. Replace the 2.x.0 with the actual name of the war 
     file without the extension, e.g. RAMS-2.2.0
     
Example: 
--------
     $0 RAMS-2.2.0

END
}

NEWAPPNAME=RAMS-2.2.0

if [ $# -eq 1 ] 
then
    if [[ "$1" == "-h" ]] || [[ "$1" == "--help" ]] || [[ "$1" == "-help" ]]
	then
		myHelp
		exit 0
	elif [ -z "$1" ]
		then 
			OLDAPPNAME=RAMS-2.1.0
			echo "INFO: Beginning backing up old application by dafaulting to $OLDAPPNAME"
		else
			OLDAPPNAME=$1
			echo "INFO: Beginning backing up old application $OLDAPPNAME"
	fi
else 
	echo "ERROR: Received $# parameters when expecting the release to deploy."
	myHelp
	exit 1
fi


NEWAPPNAME=RAMS-2.2.0

# if [ -z "$1" ]
# then 
	# OLDAPPNAME=RAMS-2.1.0
	# echo "INFO: Beginning backing up old application by dafaulting to $OLDAPPNAME"
# else
	# OLDAPPNAME=$1
	# echo "INFO: Beginning backing up old application $OLDAPPNAME"
# fi

#See if Tomcat home directory, $CATALINA_HOME, is defined and if it exists
if [ -z ${CATALINA_HOME+x} ] 
then
   echo "INFO: \$CATALINA_HOME is not set - defaulting to /usr/share/apache-tomcat-7.0.57" 
   CATALINA_HOME="/usr/share/apache-tomcat-7.0.57"
else 
   echo "INFO: using \$CATALINA_HOME environment variable value $CATALINA_HOME" 
fi

if [ -d $CATALINA_HOME ]
then
   echo "INFO: \$CATALINA_HOME found"
else 
   echo "INFO: \$CATALINA_HOME not found at $CATALINA_HOME"
fi

cd $CATALINA_HOME
if [ ! -d $CATALINA_HOME/backup ]
then 
   echo "INFO: creating \$CATALINA_HOME/backup" 
   mkdir -p $CATALINA_HOME/backup
else 
   echo "INFO: \$CATALINA_HOME/backup found"
fi

if [ ! -d $CATALINA_HOME/deploy ]
then 
   echo "INFO: creating \$CATALINA_HOME/deploy" 
   mkdir -p $CATALINA_HOME/deploy
else 
   echo "INFO: \$CATALINA_HOME/deploy found"
fi

# The output from service tomcat7 status will be something like 
# "Tomcat servlet engine is running with pid 14416." Look in the 8th word for the pid.
pid=$(service tomcat7 status | cut -d ' ' -f 8)
# If the pid is not empty stop tomcat
if [ ! -z "$pid" ]; then
	echo "INFO: stopping tomcat (pid=$pid) ..."
	service tomcat7 stop
else
	echo "INFO: tomcat is already stopped."
fi

##Alternate approach for checking if service is running
#service=replace_me_with_a_valid_service
#if (( $(ps -ef | grep -v grep | grep $service | wc -l) > 0 ))
#then
#echo "$service is running!!!"
#else
#/etc/init.d/$service start
#fi


echo "INFO: saving the RAMS-*.xml and RAMS-*.war files to $CATALINA_HOME/backup"
cp $CATALINA_HOME/conf/Catalina/localhost/RAMS-*.xml $CATALINA_HOME/backup/.
cp $CATALINA_HOME/webapps/RAMS-*.war $CATALINA_HOME/backup/.

echo "INFO: removing the RAMS-*.xml and RAMS-*.war files and the exploded war"
rm $CATALINA_HOME/conf/Catalina/localhost/RAMS-*.xml
rm $CATALINA_HOME/webapps/RAMS-*.war
rm -rf $CATALINA_HOME/webapps/RAMS-*

CODEC=commons-codec-1.9.jar
if [ -f $CATALINA_HOME/lib/$CODEC ]
then 
   echo "INFO: backing up $CODEC"
   mv $CATALINA_HOME/lib/$CODEC $CATALINA_HOME/backup/$CODEC
else 
   echo "WARNING: $CODEC not found in \$CATALINA_HOME/lib"
fi

rc=0
echo "INFO: verifying that the files are in the backup location and no longer in the deployed location"
if [ -f $CATALINA_HOME/conf/Catalina/localhost/$OLDAPPNAME.xml ]
then 
   echo "ERROR: $OLDAPPNAME.xml still exists in \$CATALINA_HOME/conf/Catalina/localhost directory"
   rc=1
fi
if [ -f $CATALINA_HOME/webapps/$OLDAPPNAME.war ]
then 
   echo "ERROR: $OLDAPPNAME.war still exists in \$CATALINA_HOME/webapps directory"
   rc=2
fi
if [ -d $CATALINA_HOME/webapps/$OLDAPPNAME ]
then 
   echo "ERROR: RAMS-* directory still exists in \$CATALINA_HOME/webapps directory"
   rc=3
fi
if [ ! -f $CATALINA_HOME/backup/$OLDAPPNAME.xml ]
then 
   echo "ERROR: $OLDAPPNAME.xml not found in \$CATALINA_HOME/backup directory"
   rc=4
fi
if [ ! -f $CATALINA_HOME/backup/$OLDAPPNAME.war ]
then 
   echo "ERROR: $OLDAPPNAME.war not found in \$CATALINA_HOME/backup directory"
   rc=5
fi
if [ ! -f $CATALINA_HOME/backup/$CODEC ]
then 
   echo "WARNING: $CODEC not found in \$CATALINA_HOME/backup directory"
   rc=6
fi

echo "INFO: Backup Done - exit code=$rc"
if [[ $rc -eq 0 ]]
then
	echo
	echo "INFO: The next step is to run sudo -E ./deploy.sh $NEWAPPNAME"
	echo
fi

exit $rc
