﻿<#
.SYNOPSIS
    Bulk import SSRS Reports from a folder and update the reports data source.
.DESCRIPTION
    This script takes all the RDL files in a folder and imports them into a SSRS and updates the reports data source to the Configmgr shared data source.
#>

Param(
   [string]$webServiceUrl,
      
   [string]$reportFolder = "RAMS Reports",

   [string]$DataSourcesFolder = "Data Sources",

   [string]$SourceDirectory = $PSScriptRoot,

   [bool]$overwrite = $true,

   # DataSources Connection string Table
   [hashtable]$rdsConnStringTable,

   # DataSource Credentials: Username and Password if needed
   [hashtable]$rdsCredentialsTable
   )

   #$rdsConnStringTable = @{ "RAMS_SP" = "http://DNS.URL/sites/IRBApplications/" ; "DataSource1" = "http://DNS.URL/sites/IRBApplications/"; "test" = "Data Source=DNS  DNS;Initial Catalog=RAMS"};
   #$rdsCredentialsTable = @{"RAMS_DB" = @("BLABLA","&35Dkc>Av+KX")};

#Connect to SSRS
Write-Host "Reportserver: $webServiceUrl" -ForegroundColor DarkMagenta
Write-Host "Estabishing Proxy connection, connecting to : $webServiceUrl/ReportServer/ReportService2010.asmx?WSDL"
Write-Host ""

$ssrsProxy = New-WebServiceProxy -Uri $webServiceUrl'/ReportServer/ReportService2010.asmx?WSDL' -UseDefaultCredential
$ssrsProxy


function SSRSFolder ([string]$reportFolder,[string]$reportPath)
{
   
#Create Folder     
        try
        {
            $ssrsProxy.CreateFolder($reportFolder, $reportPath, $null) | out-null
            if($reportPath -eq '/')
            {
            Write-Host "Folder `"$reportpath$reportFolder`" Created"

            }
            else
            {
            Write-Host "Folder `"$reportpath/$reportFolder`" Created"
            }
        }
        catch [System.Web.Services.Protocols.SoapException]
        {
            if ($_.Exception.Detail.InnerText -match "rsItemAlreadyExists400")
            {
                Write-Host "Folder: $reportFolder already exists."
            }
            else
            {
                $msg = "Error creating folder: $reportFolder. Msg: '{0}'" -f $_.Exception.Detail.InnerText
                Write-Error $msg
            }
        }
}

Function SSRSItem ([string]$ItemType,$item,[string]$folder)
 {
 Write-host ""
 #ReportName
 if ($ItemType -ne "Resource")
 {
 $ItemName = [System.IO.Path]::GetFileNameWithoutExtension($item);
 }
 else
 {
 $ItemName = $item.Name
 }
 write-host $ItemName -ForegroundColor Green 

 #Upload File
     try
    {
        #Get Report content in bytes
        Write-Host "Getting file content of : $item"
        $byteArray = Get-Content $item.FullName -encoding byte
        $msg = "Size: {0} KB" -f "{0:N0}" -f ($byteArray.Length/1KB) 
        Write-Host $msg 
        
        Write-Host "Uploading to: $folder"
 
        #Sets property for images(only png)
        $type = $ssrsProxy.GetType().Namespace
        $datatype = ($type + '.Property')              
        $property =New-Object ($datatype);
        if ($ItemType -eq "Resource" -and $item -like "*.png")
        {
        $property.Name = "MimeType"
        $property.Value = “image/png”
        }
        else
        {
        $property = $null
        }

        #Call Proxy to upload report  
        $warnings =@();    
        $ssrsProxy.CreateCatalogItem($ItemType,$itemName,$folder,$overwrite,$byteArray,$property,[ref]$warnings) | out-null
        if($warnings.Length -le 1) { Write-Host "Upload Success." -ForegroundColor Green }
        else 
        {        
            foreach ($message in $warnings)
                {
                if ($message.Code -ne "rsDataSourceReferenceNotPublished")
                    {
                    write-host "$($message.Severity) $($message.Code) $($message.Message)" -ForegroundColor Yellow
                    }
                }
        }
    }
    catch [System.IO.IOException]
    {
        $msg = "Error while reading rdl file : '{0}', Message: '{1}'" -f $rdlFile, $_.Exception.Message
        Write-Error msg
    }
    catch [System.Web.Services.Protocols.SoapException]
 {
    $caught = $false
            if ($_.Exception.Detail.InnerText -match 'rsItemAlreadyExists400')
            {
                $caught = $true
                Write-Host "Report: $itemName already exists." -ForegroundColor Red
            }
            if ($_.Exception.Detail.InnerText -match 'CoretechSSRS')
            {
                $caught = $true
                Write-Host "Cant't find Reporting Extention File." -ForegroundColor Red
            }
            elseif ($caught -eq $false)
            {
                $msg = "Error uploading report: $reportName. Msg: '{0}'" -f $_.Exception.Detail.InnerText
                Write-Error $msg
            }
  
 }
}

Function SSRSDatasource ([string]$ReportPath,$DataSourceName,$DataSourcePath)
{
$report = $ssrsProxy.GetItemDataSources($ReportPath)
ForEach ($Source in $report)
    {
    $DataSourcePath= $DataSourcePath +"/"+ $Source.Name;
    echo "in SSRSDataSource DataSourcePAth" $DataSourcePath;

    $DataSourceName =$Source.Name;
    echo "in SSRSDataSource DataSourceName" $DataSourceName;


    $proxyNamespace = $Source.GetType().Namespace
    
        $constDatasource = New-Object ("$proxyNamespace.DataSource")
        $constDatasource.Name = $DataSourceName
        $constDatasource.Item = New-Object ("$proxyNamespace.DataSourceReference")
        $constDatasource.Item.Reference = $DataSourcePath

   $Source.item = $constDatasource.Item
    $ssrsProxy.SetItemDataSources($ReportPath, $Source)
    Write-Host "Changing datasource `"$($Source.Name)`" to $($Source.Item.Reference)"
    }
}

##Create Folder Structure
#Create Base Folder
SSRSFolder $reportFolder "/"
SSRSFolder $DataSourcesFolder "/"

ForEach($Folder in Get-ChildItem $SourceDirectory -Directory)
    {
    #Add each folder in the sourcefolder to the reporting service 
    SSRSFolder $Folder.Name /$reportFolder
    
    }

    #Create the Data Sources
    ForEach ($rdsfile in Get-ChildItem $SourceDirectory -Filter *.rds )
{ 
#Get the datasource file name without extension
$rdsFileName = [System.IO.Path]::GetFileNameWithoutExtension($rdsfile)
#Source file
$src = $SourceDirectory +"/"+ $rdsfile

#Target folder
$tar = "/"+ $DataSourcesFolder

Write-Host "FOR:" $rdsFileName

# CREATE A NEW DATASOURCE OBJECT WITH SOURCE FILE DATA
[xml]$XmlDataSourceDefinition = Get-Content $src;
    

    Write-Host "Temp;late Definition"
    $XmlDataSourceDefinition.DataSourceDefinition

    try
    { 
        $type = $ssrsproxy.GetType().Namespace;
    }
    catch
    { 
        throw $_.Exception;
    }
    
    $dataSourceDefinitionType = ($type + '.DataSourceDefinition');
    $dataSourceDefinition = New-Object ($dataSourceDefinitionType);
    $dataSourceDefinition.Extension = $XmlDataSourceDefinition.DataSourceDefinition.Extension;



    #Set the connectionstring from the $rdsConnStringTable provided

    if($rdsConnStringTable.ContainsKey($rdsFileName)){

        if($rdsConnStringTable.Item($rdsFileName) -ne $null -and $rdsConnStringTable.Item($rdsFileName) -ne "" ){
        $rdsConnStringTable.Item($rdsFileName);
            $dataSourceDefinition.ConnectString = $rdsConnStringTable.Item($rdsFileName);
        }
        else{
            Write-Host "The Connection string provided against Datasource" $rdsFileName "is Null or empty" -foregroundcolor Red
        }
    }
    else{
        Write-Host "The Connection string against Datasource" $rdsFileName "has not been provided, It will be set as blank/empty going further" -foregroundcolor Red
                    $dataSourceDefinition.ConnectString = "";

    }

    $dataSourceDefinition.CredentialRetrieval = $XmlDataSourceDefinition.DataSourceDefinition.CredentialRetrieval;
    $dataSourceDefinition.WindowsCredentials = $XmlDataSourceDefinition.DataSourceDefinition.WindowsCredentials;
    $dataSourceDefinition.Enabled = $XmlDataSourceDefinition.DataSourceDefinition.Enabled;
    $dataSourceDefinition.ImpersonateUser = $XmlDataSourceDefinition.DataSourceDefinition.ImpersonateUser;
    

    # Set the authentication Credentials from the $rdsCredentialsTable

    if($rdsCredentialsTable.ContainsKey($rdsFileName)){

        if($rdsCredentialsTable.Item($rdsFileName) -ne $null -and $rdsCredentialsTable.Item($rdsFileName).count -eq 2){
            
            $dataSourceDefinition.CredentialRetrieval = 'Store';
            $dataSourceDefinition.WindowsCredentials = $true;
            $dataSourceDefinition.UserName=$rdsCredentialsTable.Item($rdsFileName)[0];
            $dataSourceDefinition.Password=$rdsCredentialsTable.Item($rdsFileName)[1];
        }
        else{
            Write-Host "The Credentials provided against Datasource" $rdsFileName "is Null or empty" -foregroundcolor Red
        }
    }
    
    Write-Host "New Definition"
    $dataSourceDefinition

    try
    { 
        #CREATE THE DATASOURCE
        $newDataSource = $ssrsproxy.CreateDataSource([System.IO.Path]::GetFileNameWithoutExtension($rdsfile),$tar,$true,$dataSourceDefinition,$null); 
    }
    catch
    { 
        throw $_.Exception; 
    }


}

     
#Upload Reports in root folder
ForEach ($rdlfile in Get-ChildItem $SourceDirectory -Filter *.rdl )
{
SSRSItem Report $rdlfile /$reportFolder
<#Change Report Datasource
    $ReportPath = "/" + $reportFolder + "/" +  [System.IO.Path]::GetFileNameWithoutExtension($rdlfile)
    $DataSourcePath = "/" + $DataSourcesFolder;
    $DatasourceName= "dummy"
    echo "ReportPath" $ReportPath
    echo "DataSourcePath" $DataSourcePath
    SSRSDatasource $ReportPath $DataSourceName $DataSourcePath
#>
}

##For Each folder
ForEach($Folder in Get-ChildItem $SourceDirectory -Directory)
    {
    #Add each folder in the sourcefolder to the reporting service 
    #SSRSFolder $Folder.Name /$Basefolder 
    
    #Add reports in the folder
    ForEach ($rdlfile in Get-ChildItem $Folder -Filter *.rdl )
        {
        SSRSItem Report $rdlfile /$reportFolder/$($folder.Name) 

        #Change Report Datasource
        $ReportPath = "/" + $reportFolder + "/" + $folder.Name + "/" + $rdlfile.BaseName 
        #SSRSDatasource $ReportPath $DataSourceName $DataSourcePath
        }
    }